//******************* OurNSHandler.pas *****************
unit OurNSHandler;
 
(* Simple demo for temporary pluggable NameSpacehandler
   To add more functionality to the namespacehandler
   take a look at the following link:
 
 http://msdn.microsoft.com/workshop/networking/pluggable/pluggable.asp
 
  For discussions about APP, namespacehandlers, 
  mimefilters and other delphi-webbrowser topics use:
 
  http://www.egroups.com/group/delphi-webbrowser/info.html
 
   Go to http://www.bsalsa.com/name_space.html for more 
   info about this sample and updated versions.
           Per Linds Larsen, Nov. 1999
*)
 
interface
uses
  {$IFDEF VER120} d4_app,{$ENDIF} Classes, Windows, 
  Forms, Axctrls, dialogs, SysUtils, ComObj, ActiveX, 
  UrlMon;
 
const
  Class_OurNSHandler: TGUID = 
      '{0EB00680-8FA1-11D3-96C7-829E3EA50C29}';
  // Create your own GUID - In Delphi IDE: Ctrl-Shift-G
 
  NameSpace = 'testprogram';
 
type
  TOurNSHandler = class(TComObject, IInternetProtocol)
  private
    Url: string;
    Written, TotalSize: Integer;
    ProtSink: IInternetProtocolSink;
    DataStream: IStream;
  protected
// IInternetProtocol Methods
    function Start(szUrl: PWideChar; 
                   OIProtSink: IInternetProtocolSink;
                   OIBindInfo: IInternetBindInfo; 
                   grfPI, dwReserved: DWORD): HResult; stdcall;
    function Continue(const ProtocolData: 
                      TProtocolData): HResult; stdcall;
    function Abort(hrReason: HResult; 
                   dwOptions: DWORD): HResult; stdcall;
    function Terminate(dwOptions: DWORD): HResult; stdcall;
    function Suspend: HResult; stdcall;
    function Resume: HResult; stdcall;
    function Read(pv: Pointer; cb: ULONG; 
                  out cbRead: ULONG): HResult; stdcall;
    function Seek(dlibMove: LARGE_INTEGER; 
                  dwOrigin: DWORD; out libNewPosition: ULARGE_INTEGER): HResult; stdcall;
    function LockRequest(dwOptions: DWORD): HResult; stdcall;
    function UnlockRequest: HResult; stdcall;
// Helper functions
    procedure GetDataFromFile(Url: string);
  end;
 
 
 
implementation
 
uses
  unit1, comserv;
 
 
function TOurNSHandler.Start(szUrl: PWideChar; OIProtSink: IInternetProtocolSink;
  OIBindInfo: IInternetBindInfo; grfPI, dwReserved: DWORD): HResult; stdcall;
begin
(* We receive all http://-URL's here and let the
default protocolhandler take over if we don't find
our namespace.
*)
  if Pos('http://' + NameSpace + '/', LowerCase(szUrl)) <> 1
  then Result := INET_E_USE_DEFAULT_PROTOCOLHANDLER
 
  else begin
  Url := SzUrl;
  written := 0;
  ProtSink := OIProtSink; //Get interface to Transaction handlers IInternetnetProtocolSink
(* Now get the data and load it in DataStream *)
 GetDataFromFile(Url);
(*Inform Transaction handler that all data is ready *)
  ProtSink.ReportData(BSCF_FIRSTDATANOTIFICATION or
        BSCF_LASTDATANOTIFICATION or BSCF_DATAFULLYAVAILABLE, TotalSize, TotalSize);
(* -> Here our Read Method is called by transaction handler*)
  ProtSink.ReportResult(S_OK, S_OK, nil);
(* Report result to transaction handler. Our Terminate method will be called *)
  Result := S_OK;
  end;
end;
 
 
function TOurNSHandler.Read(pv: Pointer; cb: ULONG; out cbRead: ULONG): HResult;
begin
(*Read Data from DataStream to Browser/URLMON *)
  DataStream.Read(pv, cb, @cbRead);
  Inc(written, cbread);
  if (written = totalSize) then  result := S_FALSE else Result := E_PENDING;
end;
 
 


Procedure TOurNSHandler.GetDataFromFile(Url: string);
var
  t: TMemoryStream;
  Dummy: INT64;
  r: TResourceStream;
begin
  delete(url, 1, 7+length(namespace)+1);// delete the "http://testprogram/"
  delete(url, length(url)-3, 4);// delete the ".mht" extension
  t:=TMemoryStream.Create;
  try
    r:=TResourceStream.Create(hinstance, Url, RT_RCDATA);
    try
      t.loadfromstream(r);
    finally
      freeandnil(r);
    end;
    Application.ProcessMessages;
    t.SaveToFile(url+'_memory');
    CreateStreamOnHGlobal(0, True, DataStream);
    TOleStream.Create(DataStream).CopyFrom(t, t.Size);
    DataStream.Seek(0, STREAM_SEEK_SET, Dummy);
    TotalSize := t.Size;
  finally
    freeandnil(t);
  end;
end;

 
 
 
function TOurNSHandler.Terminate(dwOptions: DWORD): HResult; stdcall;
begin
  DataStream._Release;
  Protsink._Release;
  result := S_OK;
end;
 
function TOurNSHandler.LockRequest(dwOptions: DWORD): HResult; stdcall;
begin
  result := S_OK;
end;
 
function TOurNSHandler.UnlockRequest: HResult;
begin
  result := S_OK;
end;
 
function TOurNSHandler.Continue(const ProtocolData: TProtocolData): HResult;
begin
  result := S_OK;
end;
 
function TOurNSHandler.Abort(hrReason: HResult; dwOptions: DWORD): HResult; stdcall;
begin
  result := E_NOTIMPL;
end;
 
function TOurNSHandler.Suspend: HResult; stdcall;
begin
  result := E_NOTIMPL;
end;
 
function TOurNSHandler.Resume: HResult; stdcall;
begin
  result := E_NOTIMPL;
end;
 
function TOurNSHandler.Seek(dlibMove: LARGE_INTEGER; dwOrigin: DWORD;
  out libNewPosition: ULARGE_INTEGER): HResult;
begin
  result := E_NOTIMPL;
end;
 
initialization
    TComObjectFactory.Create(ComServer, 
                             TOurNSHandler,   
                             Class_OurNSHandler,
                             'OurNSHandler', 
                             'OurNSHandler', 
                             ciMultiInstance, 
                             tmApartment);
 
 
end.

